<?php

declare(strict_types=1);

namespace Drupal\inline_permissions;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccessPolicyBase;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Session\CalculatedPermissionsItem;
use Drupal\Core\Session\RefinableCalculatedPermissionsInterface;

/**
 * An access policy for granting inline permissions.
 */
class InlinePermissionsAccessPolicy extends AccessPolicyBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Constructs a new InlinePermissionsAccessPolicy.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager) {
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public function calculatePermissions(AccountInterface $account, string $scope): RefinableCalculatedPermissionsInterface {
    // Get the current calculated permissions.
    $calculated_permissions = parent::calculatePermissions($account, $scope);

    // Get the user from the current account.
    $userStorage = $this->entityTypeManager->getStorage('user');
    $user = $userStorage->load($account->id());

    $cacheability = new CacheableMetadata();
    // Invalidate the cache when the user object changes.
    $cacheability->addCacheableDependency($user);

    $calculated_permissions->addCacheableDependency($cacheability);

    if ($user->inline_permissions->isEmpty()) {
      return $calculated_permissions;
    }
    // Get the field value and split it on new lines.
    // Each line will be a permission.
    $inline_permissions_value = $user->inline_permissions->value;
    $inline_permissions = preg_split("/\r\n|\n|\r/", $inline_permissions_value);

    foreach ($inline_permissions as $permission) {
      $calculated_permissions->addItem(
        item: new CalculatedPermissionsItem(
          permissions: [$permission],
          isAdmin: FALSE
        ),
      );
    }

    return $calculated_permissions;
  }

  /**
   * {@inheritdoc}
   */
  public function getPersistentCacheContexts(): array {
    return ['user'];
  }

}
